==> scrapy_elasticsearch-0.1.0-py3-none-any.whl: scrapy_elasticsearch/pipelines.py <==
# Define your item pipelines here
#
import time
import types
import logging
from datetime import datetime

from scrapy.exceptions import DropItem

from elasticsearch import Elasticsearch
from elasticsearch import helpers
from elasticsearch.exceptions import RequestError
from elasticsearch.exceptions import ConnectionTimeout
from elasticsearch.exceptions import ConnectionError
from elasticsearch.exceptions import SSLError
from elasticsearch.exceptions import TransportError


class InvalidSettingsException(Exception):
    pass


class ElasticSearchPipeline(object):
    settings = None
    es = None
    items_buffer = []
    id_key = None
    max_retry = 3

    @classmethod
    def validate_settings(cls, settings):
        def validate_setting(setting_key):
            if settings[setting_key] is None:
                raise InvalidSettingsException('%s is not defined in settings.py' % setting_key)

        required_settings = {'ELASTICSEARCH_INDEX'}

        for required_setting in required_settings:
            validate_setting(required_setting)

    @classmethod
    def init_es_client(cls, crawler_settings):
        es_timeout = crawler_settings.get('ELASTICSEARCH_TIMEOUT', 60)

        es_servers = crawler_settings.get('ELASTICSEARCH_SERVERS', 'localhost:9200')
        es_servers = es_servers if isinstance(es_servers, list) else [es_servers]

        es_settings = dict()
        es_settings['hosts'] = es_servers
        es_settings['timeout'] = es_timeout

        if crawler_settings.get('ELASTICSEARCH_USERNAME') and \
            crawler_settings.get('ELASTICSEARCH_PASSWORD'):
            es_settings['http_auth'] = (
                crawler_settings['ELASTICSEARCH_USERNAME'],
                crawler_settings['ELASTICSEARCH_PASSWORD'])

        es = Elasticsearch(**es_settings)

        return es

    @classmethod
    def from_crawler(cls, crawler):
        ext = cls()
        ext.settings = crawler.settings

        cls.validate_settings(ext.settings)
        ext.es = cls.init_es_client(crawler.settings)
        ext.max_retry = crawler.settings.get('ELASTICSEARCH_MAX_RETRY', 3)
        ext.id_key = crawler.settings.get('ELASTICSEARCH_ID_KEY')
        return ext

    def index_item(self, item):
        index_name = item.pop('indice_name', self.settings['ELASTICSEARCH_INDEX'])
        _id = item.pop('_id', None)

        index_action = {
            '_index': index_name,
            '_type': '_doc',
            '_source': dict(item)
        }

        action = item.pop('action', 'index') or 'index'
        index_action['_op_type'] = action

        if _id:
            index_action['_id'] = _id
        else:
            if self.id_key is not None and self.id_key in item:
                index_action['_id'] = item[self.id_key]

        self.items_buffer.append(index_action)

        if len(self.items_buffer) >= int(self.settings.get('ELASTICSEARCH_BUFFER_LENGTH', 10)):
            self.send_items()
            self.items_buffer = []

    def send_items(self):
        helpers.bulk(self.es, self.items_buffer)
        retry = self.max_retry
        while retry > 0:
            try:
                helpers.bulk(self.es, self.items_buffer)
                break
            except (ConnectionTimeout, ConnectionError, SSLError, TransportError):
                time.sleep(retry)
                retry -= 1
                continue
            except RequestError as e:
                break
            except Exception as e:
                raise e

    def process_item(self, item, spider):
        if isinstance(item, (types.GeneratorType, list)):
            for each in item:
                self.process_item(each, spider)
        else:
            self.index_item(item)
            logging.debug(
                'Item has been sent to Elastic Search %s',
                self.settings['ELASTICSEARCH_INDEX'])

        return item

    def close_spider(self, spider):
        if len(self.items_buffer):
            self.send_items()

==> scrapy_elasticsearch-0.1.1-py3-none-any.whl: scrapy_elasticsearch/pipelines.py <==
# Define your item pipelines here
#
import time
import types
import logging
from datetime import datetime

from scrapy.exceptions import DropItem

from elasticsearch import Elasticsearch
from elasticsearch import helpers
from elasticsearch.exceptions import RequestError
from elasticsearch.exceptions import ConnectionTimeout
from elasticsearch.exceptions import ConnectionError
from elasticsearch.exceptions import SSLError
from elasticsearch.exceptions import TransportError


class InvalidSettingsException(Exception):
    pass


class ElasticSearchPipeline(object):
    settings = None
    es = None
    items_buffer = []
    id_key = None
    max_retry = 3

    @classmethod
    def validate_settings(cls, settings):
        def validate_setting(setting_key):
            if settings[setting_key] is None:
                raise InvalidSettingsException('%s is not defined in settings.py' % setting_key)

        required_settings = {'ELASTICSEARCH_INDEX'}

        for required_setting in required_settings:
            validate_setting(required_setting)

    @classmethod
    def init_es_client(cls, crawler_settings):
        es_timeout = crawler_settings.get('ELASTICSEARCH_TIMEOUT', 60)

        es_servers = crawler_settings.get('ELASTICSEARCH_SERVERS', 'localhost:9200')
        es_servers = es_servers if isinstance(es_servers, list) else [es_servers]

        es_settings = dict()
        es_settings['hosts'] = es_servers
        es_settings['timeout'] = es_timeout

        if crawler_settings.get('ELASTICSEARCH_USERNAME') and \
            crawler_settings.get('ELASTICSEARCH_PASSWORD'):
            es_settings['http_auth'] = (
                crawler_settings['ELASTICSEARCH_USERNAME'],
                crawler_settings['ELASTICSEARCH_PASSWORD'])

        es = Elasticsearch(**es_settings)

        return es

    @classmethod
    def from_crawler(cls, crawler):
        ext = cls()
        ext.settings = crawler.settings

        cls.validate_settings(ext.settings)
        ext.es = cls.init_es_client(crawler.settings)
        ext.max_retry = crawler.settings.get('ELASTICSEARCH_MAX_RETRY', 3)
        ext.id_key = crawler.settings.get('ELASTICSEARCH_ID_KEY')
        return ext

    def index_item(self, item):
        index_name = item.pop('indice_name', self.settings['ELASTICSEARCH_INDEX'])
        _id = item.pop('_id', None)
        action = item.pop('action', 'index') or 'index'

        index_action = {
            '_index': index_name,
            '_type': '_doc',
            '_op_type': action,
        }
        if action == 'update':
            index_action['doc'] = dict(item)
        else:
            index_action['_source'] = dict(item)

        if _id:
            index_action['_id'] = _id
        else:
            if self.id_key is not None and self.id_key in item:
                index_action['_id'] = item[self.id_key]

        self.items_buffer.append(index_action)

        if len(self.items_buffer) >= int(self.settings.get('ELASTICSEARCH_BUFFER_LENGTH', 10)):
            self.send_items()
            self.items_buffer = []

    def send_items(self):
        helpers.bulk(self.es, self.items_buffer)
        retry = self.max_retry
        while retry > 0:
            try:
                helpers.bulk(self.es, self.items_buffer)
                break
            except (ConnectionTimeout, ConnectionError, SSLError, TransportError):
                time.sleep(retry)
                retry -= 1
                continue
            except RequestError as e:
                break
            except Exception as e:
                raise e

    def process_item(self, item, spider):
        if isinstance(item, (types.GeneratorType, list)):
            for each in item:
                self.process_item(each, spider)
        else:
            self.index_item(item)
            logging.debug(
                'Item has been sent to Elastic Search %s',
                self.settings['ELASTICSEARCH_INDEX'])

        return item

    def close_spider(self, spider):
        if len(self.items_buffer):
            self.send_items()
